This library contains a collection of basic disk operations and a key-value store.

The file format for the read-only database is:
 - The length of the JSON structure. A single number in JSON format, ended with a space.
 - A JSON structure with the file name, position and length. The key is the filename, the object contains the position and length.
 - The file bytes

The file format for the key-value database is:
 - The first block contains the header structure. It has:
   - the directory size
   - the active directory
   - the first free byte
 - There are two JSON headers. When a change occurs, write the other. Then, change the offset in an atomic write to the first block.
 - Each JSON header has enough capacity for a lot of files.
 - CRUD:
    - Create: Append the file.
    - Read: Read the file.
    - Update: Constructed out of a create and a delete.
    - Delete: Remove the file.
 - How it preserves qualities:
   - Removing a file causes the block to be marked as free.
   - Adjacent free blocks are combined.
   - Free blocks at the end are removed, and the free position is updated.
   - Adding a new file will prefer a freed block if available.
   - The disk is kept consistent all the time. Changes are only done to unused segments. Except the header, which is written as an atomic operation.
   - Can add and remove data without wasting space.
 - Qualities:
   - Can add and remove data without wasting space.
   - Operations are atomic. i.e. can survive powerloss.
   - Cannot write new data if directory or disk is full.

Transactions:
 - Serializable transactions are achieved by simply queueing workloads and doing one task after the other. This does not solve the problem of power failure in the middle.

## API

### Initialize Key Value Store
{
    "command": "Initialize",
    "directorySize": 1000
}

### Add Key Value
{
    "command": "Add",
    "key": "key",
    "value": "base64"
}

### Read Value
{
    "command": "Read",
    "key": "key"
}

### Remove Key Value
{
    "command": "Remove",
    "key": "key"
}

### Update Value of key
{
    "command": "Update",
    "key": "key",
    "value": "base64"
}

### List Entries
{
    "command": "List"
}

### List Entries Starting With
{
    "command": "ListStartingWith",
    "prefix": "Prefix"
}

### Free
{
    "command": "Free"
}

## Design idea for using a key-value store as the only database

 * Identify the main object, and store all information in these main objects. For example, in an cloud provider, the customer objects contains all the services they currently have and invoices. The whole main object is updated atomically. Also the main object is simply stored as JSON. These make the code to read and write very easy. Also, this is easy to horizontally scale. Different main objects can be places at different nodes. The search in these main objects can either be done in parallel, or a read-only view can be generated regularily, giving the system eventual consistency.

 * This idea is interesting, as it points towards why the relational model requires so much transactions (because the data is spread out in tables) , why it is difficult to horizontally scale (because it does not identify main objects), and why it grows slow (because getting a single main object requires searching in many shared tables).