package BasicFileSystem.KeyValueReadOnly;

import Bytes.Bytes.ByteArray;
import Bytes.Bytes.ByteArrayReference;
import DataStructures.Array.Structures.Data;
import DataStructures.Array.Structures.DataReference;
import DataStructures.Array.Structures.Structure;
import Disk.Disk.DiskStructure;
import ProcessingUnit.ProcessingUnitServer.ProcessingUnitServerStructure;
import references.references.*;

import static Bytes.Bytes.Bytes.*;
import static DataStructures.Array.Structures.Structures.*;
import static Disk.Disk.Disk.Entries;
import static Disk.Disk.Disk.Read;
import static DiskRanges.DiskRanges.DiskRanges.*;
import static JSON.Parser.Parser.ReadJSON;
import static JSON.Writer.Writer.WriteJSON;
import static ProcessingUnit.ProcessingUnitServer.ProcessingUnitServer.ServerReceive;
import static ProcessingUnit.ProcessingUnitServer.ProcessingUnitServer.ServerSend;
import static Unicode.Unicode.Unicode.UTF16ToUTF8;
import static Unicode.Unicode.Unicode.UTF8ToUTF16;
import static arrays.arrays.arrays.CopyNumberArrayRange;
import static arrays.arrays.arrays.NumberArrayToString;
import static arrays.arrays.arrays.StringToNumberArray;
import static nnumbers.NumberToString.NumberToString.nCreateStringDecimalFromNumber;
import static nnumbers.StringToNumber.StringToNumber.nCreateNumberFromDecimalString;
import static references.references.references.*;

public class KeyValueReadOnly {
    public static void CreateKeyValueStore(DiskStructure disk, StringReference [] keys, ByteArray [] values){
        Data root, pos, len;
        Structure fp;
        double i, j, jsonStart, currentPosition, prevLen, prevPos, firstPos;
        char[] json, jsonLengthString;
        boolean done;
        ByteArray data;

        root = CreateStructData();

        for(i = 0d; i < keys.length; i = i + 1d){
            fp = CreateStructure();
            AddNumberToStruct(fp, "pos".toCharArray(), 0d);
            AddNumberToStruct(fp, "len".toCharArray(), ByteArrayLength(values[(int) i]));
            AddStructToStruct(root.structure, keys[(int)i].string, fp);
        }

        // Compute lengths:
        prevLen = 0d;
        prevPos = 0d;
        done = false;
        for(j = 0d; !done; j = j + 1d) {
            json = WriteJSON(root);
            jsonLengthString = nCreateStringDecimalFromNumber(json.length);
            jsonStart = jsonLengthString.length + 1d;
            currentPosition = jsonStart + json.length;
            firstPos = currentPosition;

            for (i = 0d; i < keys.length; i = i + 1d) {
                fp = GetStructFromStruct(root.structure, keys[(int) i].string);
                pos = GetDataFromStruct(fp, "pos".toCharArray());
                pos.number = currentPosition;
                len = GetDataFromStruct(fp, "len".toCharArray());
                len.number = ByteArrayLength(values[(int) i]);
                currentPosition = currentPosition + ByteArrayLength(values[(int) i]);
            }

            if(json.length == prevLen && prevPos == firstPos){
                done = true;
            }else{
                prevLen = json.length;
                prevPos = firstPos;
            }
        }

        // Write contents
        currentPosition = 0d;

        json = WriteJSON(root);
        jsonLengthString = nCreateStringDecimalFromNumber(json.length);
        data = UTF16ToUTF8(jsonLengthString);
        WriteRange(disk, currentPosition, jsonLengthString.length, data);

        currentPosition = currentPosition + jsonLengthString.length;

        data = UTF16ToUTF8(" ".toCharArray());
        WriteRange(disk, jsonLengthString.length, 1d, data);

        currentPosition = currentPosition + 1d;

        data = UTF16ToUTF8(json);
        WriteRange(disk, currentPosition, json.length, data);

        currentPosition = currentPosition + json.length;

        for(i = 0d; i < keys.length; i = i + 1d){
            data = values[(int)i];
            WriteRange(disk, currentPosition, ByteArrayLength(data), data);
            currentPosition = currentPosition + ByteArrayLength(data);
        }

        //System.out.println((long)currentPosition/1024);
    }

    public static Structure ReadDirectory(DiskStructure disk){
        boolean success, done;
        char c;
        double e, i, jsonLengthX;
        ByteArrayReference block, data, jsonLengthNumberArray;
        NumberReference entries, entrySize;
        char[] jsonLengthString, json;
        DataReference er;
        StringReference message;

        entries = new NumberReference();
        entrySize = new NumberReference();
        Entries(disk, entries, entrySize);
        e = entrySize.numberValue;

        block = new ByteArrayReference();
        block.bytes = CreateAndFillByteArray(e, 0d);

        // Get jsonLength
        Read(disk, 0d, block);
        done = false;
        for(i = 0d; i < e && !done; i = i + 1d){
            c = (char)GetByte(block.bytes, i);
            if(c == ' '){
                done = true;
            }
        }

        jsonLengthNumberArray = new ByteArrayReference();
        jsonLengthNumberArray.bytes = CreateByteArray(i - 1d);
        CopyByteArrayRange(block.bytes, 0d, i - 1d, jsonLengthNumberArray.bytes);
        jsonLengthString = UTF8ToUTF16(jsonLengthNumberArray.bytes);
        jsonLengthX = nCreateNumberFromDecimalString(jsonLengthString);

        data = new ByteArrayReference();
        ReadRange(disk, i, jsonLengthX, data);
        json = UTF8ToUTF16(data.bytes);
        er = new DataReference();
        message = new StringReference();
        success = ReadJSON(json, er, message);

        return er.data.structure;
    }

    public static boolean GetValue(DiskStructure disk, Structure directory, char[] key, ByteArrayReference file) {
        Data e;
        Structure f;
        double pos, len;
        BooleanReference foundRef;
        boolean success;

        foundRef = new BooleanReference();
        e = GetDataFromStructWithCheck(directory, key, foundRef);
        if(foundRef.booleanValue) {
            f = e.structure;
            pos = GetNumberFromStruct(f, "pos".toCharArray());
            len = GetNumberFromStruct(f, "len".toCharArray());

            success = ReadRange(disk, pos, len, file);
        }else{
            success = false;
        }

        return success;
    }

    static public boolean BasicFileSystemInitialize(ProcessingUnitServerStructure server, DiskStructure disk, BasicFileSystemState state){
        state.directory = ReadDirectory(disk);

        return true;
    }

    static public void BasicFileSystemIteration(ProcessingUnitServerStructure server, DiskStructure disk, BasicFileSystemState state){
        ByteArrayReference request, file;
        ByteArray response;
        char [] filename;
        boolean success;

        request = new ByteArrayReference();
        response = new ByteArray();
        file = new ByteArrayReference();

        ServerReceive(server, request);

        filename = UTF8ToUTF16(request.bytes);
        success = GetValue(disk, state.directory, filename, file);

        if(success){

        }else{
            file.bytes = CreateByteArray(0d);
        }

        ServerSend(server, file.bytes);

        delete(file);
        delete(request);
        delete(response);
    }

    public static void delete(Object o) {
        // Java has garbage collection.
    }
}























