package BasicFileSystem.KeyValueStore;

import Bytes.Bytes.ByteArray;
import Bytes.Bytes.ByteArrayReference;
import DataStructures.Array.Structures.Array;
import DataStructures.Array.Structures.Data;
import DataStructures.Array.Structures.DataReference;
import Disk.Disk.DiskStructure;
import ProcessingUnit.ProcessingUnitServer.ProcessingUnitServerStructure;
import references.references.NumberArrayReference;
import references.references.NumberReference;
import references.references.StringArrayReference;
import references.references.StringReference;

import static BasicFileSystem.KeyValue.KeyValue.*;
import static DataStructures.Array.Arrays.Arrays.CreateArray;
import static DataStructures.Array.Structures.Structures.*;
import static JSON.Parser.Parser.ReadJSON;
import static JSON.Writer.Writer.WriteJSON;
import static ProcessingUnit.ProcessingUnitServer.ProcessingUnitServer.ServerReceive;
import static ProcessingUnit.ProcessingUnitServer.ProcessingUnitServer.ServerSend;
import static Unicode.Unicode.Unicode.UTF16ToUTF8;
import static Unicode.Unicode.Unicode.UTF8ToUTF16;
import static arrays.arrays.arrays.*;
import static base64.Base64Decoder.Base64Decoder.Base64Decode;
import static base64.Base64Encoder.Base64Encoder.Base64Encode;
import static references.references.references.CreateStringArrayReferenceLengthValue;

public class KeyValueStore {
    static public boolean KeyValueStoreInitialize(ProcessingUnitServerStructure server, DiskStructure disk, KeyValueStoreState state){
        return true;
    }

    static public void KeyValueStoreIteration(ProcessingUnitServerStructure server, DiskStructure disk, KeyValueStoreState state){
        ByteArrayReference request;
        ByteArray response;
        ByteArrayReference value;
        char [] json, command, message, jsonResponse, key, valueBase64, prefix;
        boolean success;
        DataReference req;
        Data resp;
        StringArrayReference dir;
        double directorySize, i;
        NumberReference diskSpace, directorySpace;
        StringReference messageRef, base64Ref;
        Array list;

        request = new ByteArrayReference();
        base64Ref = new StringReference();
        resp = CreateStructData();
        req = new DataReference();
        message = "".toCharArray();
        messageRef = new StringReference();

        ServerReceive(server, request);
        json = UTF8ToUTF16(request.bytes);
        success = ReadJSON(json, req, messageRef);

        if(success){
            if(StructHasKey(req.data.structure, "command".toCharArray())) {
                command = GetStringFromStruct(req.data.structure, "command".toCharArray());

                if (StringsEqual(command, "Initialize".toCharArray())) {
                    if(StructHasKey(req.data.structure, "directorySize".toCharArray())) {
                        directorySize = GetNumberFromStruct(req.data.structure, "directorySize".toCharArray());
                        success = InitializeKeyValueStore(disk, directorySize);
                    }else{
                        success = false;
                        message = "Initialize command must include directorySize.".toCharArray();
                    }
                } else if (StringsEqual(command, "Add".toCharArray())) {
                    if(StructHasKey(req.data.structure, "key".toCharArray()) && StructHasKey(req.data.structure, "value".toCharArray())) {
                        key = GetStringFromStruct(req.data.structure, "key".toCharArray());
                        valueBase64 = GetStringFromStruct(req.data.structure, "value".toCharArray());
                        value = new ByteArrayReference();
                        success = Base64Decode(valueBase64, value, messageRef);
                        if(success) {
                            success = KeyValueAdd(disk, key, value.bytes, messageRef);
                            if(success){
                            }else{
                                message = messageRef.string;
                            }
                        }else{
                            message = messageRef.string;
                        }
                    }else{
                        success = false;
                        message = "Add command must include key and value.".toCharArray();
                    }
                } else if (StringsEqual(command, "Update".toCharArray())) {
                    if(StructHasKey(req.data.structure, "key".toCharArray()) && StructHasKey(req.data.structure, "value".toCharArray())) {
                        key = GetStringFromStruct(req.data.structure, "key".toCharArray());
                        valueBase64 = GetStringFromStruct(req.data.structure, "value".toCharArray());
                        value = new ByteArrayReference();
                        success = Base64Decode(valueBase64, value, messageRef);
                        if(success) {
                            success = KeyValueUpdate(disk, key, value.bytes, messageRef);
                            if(success){
                            }else{
                                message = messageRef.string;
                            }
                        }else{
                            message = messageRef.string;
                        }
                    }else{
                        success = false;
                        message = "Update command must include key and value.".toCharArray();
                    }
                } else if (StringsEqual(command, "Read".toCharArray())) {
                    if(StructHasKey(req.data.structure, "key".toCharArray())) {
                        key = GetStringFromStruct(req.data.structure, "key".toCharArray());
                        value = new ByteArrayReference();
                        success = KeyValueRead(disk, key, value, messageRef);
                        if(success){
                            success = Base64Encode(value.bytes, base64Ref, messageRef);
                            if(success){
                                AddStringToStruct(resp.structure, "value".toCharArray(), base64Ref.string);
                            }else{
                                message = messageRef.string;
                            }
                        }else{
                            message = messageRef.string;
                        }
                    }else{
                        success = false;
                        message = "Read command must include key.".toCharArray();
                    }
                } else if (StringsEqual(command, "Remove".toCharArray())) {
                    if(StructHasKey(req.data.structure, "key".toCharArray())) {
                        key = GetStringFromStruct(req.data.structure, "key".toCharArray());
                        success = KeyValueRemove(disk, key, messageRef);
                        if(success){
                        }else{
                            message = messageRef.string;
                        }
                    }else{
                        success = false;
                        message = "Remove command must include key.".toCharArray();
                    }
                } else if (StringsEqual(command, "List".toCharArray())) {
                    dir = new StringArrayReference();
                    success = KeyValueList(disk, dir, messageRef);
                    if(success){
                        list = CreateArray();
                        for(i = 0d; i < dir.stringArray.length; i = i + 1d){
                            AddStringToArray(list, dir.stringArray[(int)i].string);
                        }
                        AddArrayToStruct(resp.structure, "list".toCharArray(), list);
                    }else{
                        message = messageRef.string;
                    }
                } else if (StringsEqual(command, "ListStartingWith".toCharArray())) {
                    if(StructHasKey(req.data.structure, "prefix".toCharArray())) {
                        prefix = GetStringFromStruct(req.data.structure, "prefix".toCharArray());
                        dir = new StringArrayReference();
                        success = KeyValueListStartsWith(disk, prefix, dir, messageRef);
                        if(success){
                            list = CreateArray();
                            for(i = 0d; i < dir.stringArray.length; i = i + 1d){
                                AddStringToArray(list, dir.stringArray[(int)i].string);
                            }
                            AddArrayToStruct(resp.structure, "list".toCharArray(), list);
                        }else{
                            message = messageRef.string;
                        }
                    }else{
                        success = false;
                        message = "ListStartingWith command must include prefix.".toCharArray();
                    }
                } else if (StringsEqual(command, "Free".toCharArray())) {
                    dir = new StringArrayReference();
                    diskSpace = new NumberReference();
                    directorySpace = new NumberReference();
                    success = KeyValueFree(disk, diskSpace, directorySpace, messageRef);
                    if(success){
                        AddNumberToStruct(resp.structure, "diskSpace".toCharArray(), diskSpace.numberValue);
                        AddNumberToStruct(resp.structure, "directorySpace".toCharArray(), directorySpace.numberValue);
                    }else{
                        message = messageRef.string;
                    }
                } else {
                    success = false;
                    message = "Unknown command".toCharArray();
                }
            } else {
                success = false;
                message = "Request must contain command.".toCharArray();
            }

        }else{
            success = false;
            message = "Invalid request.".toCharArray();
        }

        AddBooleanToStruct(resp.structure, "success".toCharArray(), success);
        AddStringToStruct(resp.structure, "message".toCharArray(), message);
        jsonResponse = WriteJSON(resp);
        response = UTF16ToUTF8(jsonResponse);

        ServerSend(server, response);

        delete(request);
        delete(response);
    }

    public static void delete(Object o) {
        // Java has garbage collection.
    }
}
