package tests;

import Bytes.Bytes.ByteArray;
import Bytes.Bytes.ByteArrayReference;
import DataStructures.Array.Structures.Data;
import DataStructures.Array.Structures.Structure;
import Disk.Disk.DiskStructure;
import references.references.*;

import static BasicFileSystem.KeyValue.KeyValue.*;
import static BasicFileSystem.KeyValueReadOnly.KeyValueReadOnly.CreateKeyValueStore;
import static BasicFileSystem.KeyValueReadOnly.KeyValueReadOnly.GetValue;
import static BasicFileSystem.KeyValueReadOnly.KeyValueReadOnly.ReadDirectory;
import static Bytes.Bytes.Bytes.*;
import static DataStructures.Array.Structures.Structures.*;
import static DiskRanges.DiskRanges.DiskRanges.ReadRange;
import static Unicode.Unicode.Unicode.UTF16ToUTF8;
import static Unicode.Unicode.Unicode.UTF8ToUTF16;
import static nnumbers.NumberToString.NumberToString.nCreateStringDecimalFromNumber;
import static references.references.references.CreateNumberReference;
import static testing.testing.testing.*;

public class tests {
    public static double test(){
        NumberReference failures;

        failures = CreateNumberReference(0d);

        TestKeyValueReadOnly(failures);
        TestKeyValueReadOnly2(failures);
        TestKeyValue1(failures);
        TestKeyValue2(failures);
        TestKeyValue3(failures);
        TestKeyValueLarge(failures);

        return failures.numberValue;
    }

    public static void TestKeyValueReadOnly(NumberReference failures) {
        DiskStructure disk;
        ByteArray [] values;
        ByteArrayReference file;
        StringReference [] keys;
        Structure directory;
        boolean success;
        char [] filecontents, key;
        double i;
        Data e;

        disk = new DiskStructure();
        disk.entries = 1000d;
        disk.entrysize = 512d;
        disk.data = CreateByteArray(disk.entries * disk.entrysize);

        keys = new StringReference[3];
        keys[0] = new StringReference();
        keys[0].string = "test1".toCharArray();
        keys[1] = new StringReference();
        keys[1].string = "test2".toCharArray();
        keys[2] = new StringReference();
        keys[2].string = "test3".toCharArray();

        values = new ByteArray[3];
        values[0] = UTF16ToUTF8("hei1".toCharArray());
        values[1] = UTF16ToUTF8("hei2".toCharArray());
        values[2] = UTF16ToUTF8("hei3".toCharArray());

        CreateKeyValueStore(disk, keys, values);

        /*for(int i = 0; i < disk.data.length && i < 1000; i++){
            System.out.print((char)disk.data[i] + " ");
            if(i % 10 == 9){
                System.out.println();
            }
        }*/

        directory = ReadDirectory(disk);
        file = new ByteArrayReference();
        success = GetValue(disk, directory, "test2".toCharArray(), file);

        AssertTrue(success, failures);

        if(success){
            filecontents = UTF8ToUTF16(file.bytes);
            AssertStringEquals(filecontents, "hei2".toCharArray(), failures);
        }

        keys = GetStructKeys(directory);
        for(i = 0d; i < keys.length; i = i + 1d){
            key = keys[(int)i].string;
            e = GetDataFromStruct(directory, key);
            //System.out.println(new String(key) + " ... " + (int)GetObjectValue(e.object, "len".toCharArray()).number);
        }
    }

    public static void TestKeyValueReadOnly2(NumberReference failures) {
        DiskStructure disk;
        ByteArray [] values;
        ByteArrayReference file;
        StringReference [] keys;
        Structure directory;
        double i;
        boolean success;

        disk = new DiskStructure();
        disk.entries = 1000d;
        disk.entrysize = 512d;
        disk.data = CreateByteArray(disk.entries * disk.entrysize);

        keys = new StringReference[1];
        keys[0] = new StringReference();
        keys[0].string = "test1".toCharArray();

        values = new ByteArray[1];
        values[0] = CreateByteArray(2000d);

        for(i = 0d; i < 2000d; i = i + 1d){
            if(i % 3d == 0d) {
                SetByte(values[0], i, '1');
            }
            if(i % 3d == 1d) {
                SetByte(values[0], i, '2');
            }
            if(i % 3d == 2d) {
                SetByte(values[0], i, '3');
            }
        }

        CreateKeyValueStore(disk, keys, values);

        directory = ReadDirectory(disk);
        file = new ByteArrayReference();
        success = GetValue(disk, directory, "test1".toCharArray(), file);

        AssertTrue(success, failures);

        AssertEquals(ByteArrayLength(file.bytes), 2000d, failures);

        for(i = 0d; i < 2000d; i = i + 1d){
            if(i % 3d == 0d) {
                AssertEquals(GetByte(file.bytes, i), '1', failures);
            }
            if(i % 3d == 1d) {
                AssertEquals(GetByte(file.bytes, i), '2', failures);
            }
            if(i % 3d == 2d) {
                AssertEquals(GetByte(file.bytes, i), '3', failures);
            }
        }
    }

    public static void TestKeyValue1(NumberReference failures) {
        DiskStructure disk;
        ByteArrayReference data;
        char [] str, expected;
        ByteArray value, value2, value3;
        StringReference message;
        boolean success;
        double directorySize;
        StringArrayReference dir;
        NumberReference diskSpace, directorySpace;

        message = new StringReference();
        diskSpace = new NumberReference();
        directorySpace = new NumberReference();

        disk = new DiskStructure();
        disk.entries = 1000d;
        disk.entrysize = 128d;
        disk.data = CreateByteArray(disk.entries * disk.entrysize);
        directorySize = 200d;

        success = InitializeKeyValueStore(disk, directorySize);
        AssertTrue(success, failures);

        data = new ByteArrayReference();
        ReadRange(disk, 0d, disk.entrysize + 2d*directorySize, data);
        str = UTF8ToUTF16(data.bytes);

        expected = "{\"DirectorySize\":200,\"ActiveDirectory\":128,\"FreePosition\":528}                                                                  {\"keys\":{},\"free\":[]}                                                                                                                                                                                   {\"keys\":{},\"free\":[]}                                                                                                                                                                                   ".toCharArray();
        AssertStringEquals(expected, str, failures);

        // Free
        success = KeyValueFree(disk, diskSpace, directorySpace, message);
        AssertTrue(success, failures);
        if(success){
            AssertEquals(diskSpace.numberValue, disk.entrysize*(disk.entries - 1d) - 2d*directorySize, failures);
            AssertEquals(directorySpace.numberValue, directorySize - "{\"keys\":{},\"free\":[]}".toCharArray().length, failures);
        }

        // 1
        value = UTF16ToUTF8("testdata".toCharArray());
        success = KeyValueAdd(disk, "test".toCharArray(), value, message);

        AssertTrue(success, failures);

        data = new ByteArrayReference();
        ReadRange(disk, 0d, disk.entrysize + 2d*directorySize + ByteArrayLength(value), data);
        str = UTF8ToUTF16(data.bytes);

        expected = "{\"DirectorySize\":200,\"ActiveDirectory\":328,\"FreePosition\":536}                                                                  {\"keys\":{},\"free\":[]}                                                                                                                                                                                   {\"keys\":{\"test\":[528,8]},\"free\":[]}                                                                                                                                                                     testdata".toCharArray();
        AssertStringEquals(expected, str, failures);

        // Read
        success = KeyValueRead(disk, "test".toCharArray(), data, message);
        AssertTrue(success, failures);
        if(success) {
            str = UTF8ToUTF16(data.bytes);
            AssertStringEquals("testdata".toCharArray(), str, failures);
        }
        success = KeyValueRead(disk, "test2".toCharArray(), data, message);
        AssertFalse(success, failures);

        // 2
        value2 = UTF16ToUTF8("test2data2".toCharArray());
        success = KeyValueAdd(disk,"test2".toCharArray(), value2, message);

        AssertTrue(success, failures);

        data = new ByteArrayReference();
        ReadRange(disk, 0d, disk.entrysize + 2d*directorySize + ByteArrayLength(value) + ByteArrayLength(value2), data);
        str = UTF8ToUTF16(data.bytes);

        expected = "{\"DirectorySize\":200,\"ActiveDirectory\":128,\"FreePosition\":546}                                                                  {\"keys\":{\"test\":[528,8],\"test2\":[536,10]},\"free\":[]}                                                                                                                                                    {\"keys\":{\"test\":[528,8]},\"free\":[]}                                                                                                                                                                     testdatatest2data2".toCharArray();
        AssertStringEquals(expected, str, failures);

        // 3
        value3 = UTF16ToUTF8("3datax".toCharArray());
        success = KeyValueAdd(disk,"d3".toCharArray(), value3, message);

        AssertTrue(success, failures);

        data = new ByteArrayReference();
        ReadRange(disk, 0d, disk.entrysize + 2d*directorySize + ByteArrayLength(value) + ByteArrayLength(value2) + ByteArrayLength(value3), data);
        str = UTF8ToUTF16(data.bytes);

        expected = "{\"DirectorySize\":200,\"ActiveDirectory\":328,\"FreePosition\":552}                                                                  {\"keys\":{\"test\":[528,8],\"test2\":[536,10]},\"free\":[]}                                                                                                                                                    {\"keys\":{\"test\":[528,8],\"test2\":[536,10],\"d3\":[546,6]},\"free\":[]}                                                                                                                                       testdatatest2data23datax".toCharArray();
        AssertStringEquals(expected, str, failures);

        // List
        dir = new StringArrayReference();
        success = KeyValueList(disk, dir, message);
        AssertTrue(success, failures);
        if(success){
            AssertEquals(dir.stringArray.length,  3d, failures);
            AssertStringEquals(dir.stringArray[0].string, "test".toCharArray(), failures);
            AssertStringEquals(dir.stringArray[1].string, "test2".toCharArray(), failures);
            AssertStringEquals(dir.stringArray[2].string, "d3".toCharArray(), failures);
        }

        dir = new StringArrayReference();
        success = KeyValueListStartsWith(disk, "test".toCharArray(), dir, message);
        AssertTrue(success, failures);
        if(success){
            AssertEquals(dir.stringArray.length,  2d, failures);
            AssertStringEquals(dir.stringArray[0].string, "test".toCharArray(), failures);
            AssertStringEquals(dir.stringArray[1].string, "test2".toCharArray(), failures);
        }

        // 4
        success = KeyValueRemove(disk, "d3".toCharArray(), message);

        AssertTrue(success, failures);

        data = new ByteArrayReference();
        ReadRange(disk, 0d, disk.entrysize + 2d*directorySize + ByteArrayLength(value) + ByteArrayLength(value2) + ByteArrayLength(value3), data);
        str = UTF8ToUTF16(data.bytes);

        expected = "{\"DirectorySize\":200,\"ActiveDirectory\":128,\"FreePosition\":546}                                                                  {\"keys\":{\"test\":[528,8],\"test2\":[536,10]},\"free\":[]}                                                                                                                                                    {\"keys\":{\"test\":[528,8],\"test2\":[536,10],\"d3\":[546,6]},\"free\":[]}                                                                                                                                       testdatatest2data23datax".toCharArray();
        AssertStringEquals(expected, str, failures);

        // 5
        success = KeyValueRemove(disk, "test2".toCharArray(), message);
        AssertTrue(success, failures);

        data = new ByteArrayReference();
        ReadRange(disk, 0d, disk.entrysize + 2d*directorySize + ByteArrayLength(value) + ByteArrayLength(value2) + ByteArrayLength(value3), data);
        str = UTF8ToUTF16(data.bytes);

        expected = "{\"DirectorySize\":200,\"ActiveDirectory\":328,\"FreePosition\":536}                                                                  {\"keys\":{\"test\":[528,8],\"test2\":[536,10]},\"free\":[]}                                                                                                                                                    {\"keys\":{\"test\":[528,8]},\"free\":[]}                                                                                                                                                                     testdatatest2data23datax".toCharArray();
        AssertStringEquals(expected, str, failures);
    }

    public static void TestKeyValue2(NumberReference failures) {
        DiskStructure disk;
        ByteArrayReference data;
        char [] str, expected;
        ByteArray value, value2, value3, value4, value6;
        StringReference message;
        boolean success;
        double directorySize;

        message = new StringReference();

        disk = new DiskStructure();
        disk.entries = 1000d;
        disk.entrysize = 128d;
        disk.data = CreateByteArray(disk.entries * disk.entrysize);
        directorySize = 200d;

        success = InitializeKeyValueStore(disk, directorySize);
        AssertTrue(success, failures);

        data = new ByteArrayReference();
        ReadRange(disk, 0d, disk.entrysize + 2d*directorySize, data);
        str = UTF8ToUTF16(data.bytes);

        expected = "{\"DirectorySize\":200,\"ActiveDirectory\":128,\"FreePosition\":528}                                                                  {\"keys\":{},\"free\":[]}                                                                                                                                                                                   {\"keys\":{},\"free\":[]}                                                                                                                                                                                   ".toCharArray();
        AssertStringEquals(expected, str, failures);

        // 1
        value = UTF16ToUTF8("testdata".toCharArray());
        success = KeyValueAdd(disk, "test".toCharArray(), value, message);

        AssertTrue(success, failures);

        data = new ByteArrayReference();
        ReadRange(disk, 0d, disk.entrysize + 2d*directorySize + ByteArrayLength(value), data);
        str = UTF8ToUTF16(data.bytes);

        expected = "{\"DirectorySize\":200,\"ActiveDirectory\":328,\"FreePosition\":536}                                                                  {\"keys\":{},\"free\":[]}                                                                                                                                                                                   {\"keys\":{\"test\":[528,8]},\"free\":[]}                                                                                                                                                                     testdata".toCharArray();
        AssertStringEquals(expected, str, failures);

        // 2
        value2 = UTF16ToUTF8("test2data2".toCharArray());
        success = KeyValueAdd(disk,"test2".toCharArray(), value2, message);

        AssertTrue(success, failures);

        data = new ByteArrayReference();
        ReadRange(disk, 0d, disk.entrysize + 2d*directorySize + ByteArrayLength(value) + ByteArrayLength(value2), data);
        str = UTF8ToUTF16(data.bytes);

        expected = "{\"DirectorySize\":200,\"ActiveDirectory\":128,\"FreePosition\":546}                                                                  {\"keys\":{\"test\":[528,8],\"test2\":[536,10]},\"free\":[]}                                                                                                                                                    {\"keys\":{\"test\":[528,8]},\"free\":[]}                                                                                                                                                                     testdatatest2data2".toCharArray();
        AssertStringEquals(expected, str, failures);

        // 3
        value3 = UTF16ToUTF8("3datax".toCharArray());
        success = KeyValueAdd(disk,"d3".toCharArray(), value3, message);

        AssertTrue(success, failures);

        data = new ByteArrayReference();
        ReadRange(disk, 0d, disk.entrysize + 2d*directorySize + ByteArrayLength(value) + ByteArrayLength(value2) + ByteArrayLength(value3), data);
        str = UTF8ToUTF16(data.bytes);

        expected = "{\"DirectorySize\":200,\"ActiveDirectory\":328,\"FreePosition\":552}                                                                  {\"keys\":{\"test\":[528,8],\"test2\":[536,10]},\"free\":[]}                                                                                                                                                    {\"keys\":{\"test\":[528,8],\"test2\":[536,10],\"d3\":[546,6]},\"free\":[]}                                                                                                                                       testdatatest2data23datax".toCharArray();
        AssertStringEquals(expected, str, failures);

        // 4
        success = KeyValueRemove(disk, "test2".toCharArray(), message);

        AssertTrue(success, failures);

        data = new ByteArrayReference();
        ReadRange(disk, 0d, disk.entrysize + 2d*directorySize + ByteArrayLength(value) + ByteArrayLength(value2) + ByteArrayLength(value3), data);
        str = UTF8ToUTF16(data.bytes);

        expected = "{\"DirectorySize\":200,\"ActiveDirectory\":128,\"FreePosition\":552}                                                                  {\"keys\":{\"test\":[528,8],\"d3\":[546,6]},\"free\":[[536,10]]}                                                                                                                                                {\"keys\":{\"test\":[528,8],\"test2\":[536,10],\"d3\":[546,6]},\"free\":[]}                                                                                                                                       testdatatest2data23datax".toCharArray();
        AssertStringEquals(expected, str, failures);

        // 4
        value4 = UTF16ToUTF8("4dat".toCharArray());
        success = KeyValueAdd(disk, "test3".toCharArray(), value4, message);

        AssertTrue(success, failures);

        data = new ByteArrayReference();
        ReadRange(disk, 0d, disk.entrysize + 2d*directorySize + ByteArrayLength(value) + ByteArrayLength(value2) + ByteArrayLength(value3), data);
        str = UTF8ToUTF16(data.bytes);

        expected = "{\"DirectorySize\":200,\"ActiveDirectory\":328,\"FreePosition\":552}                                                                  {\"keys\":{\"test\":[528,8],\"d3\":[546,6]},\"free\":[[536,10]]}                                                                                                                                                {\"keys\":{\"test\":[528,8],\"d3\":[546,6],\"test3\":[536,4]},\"free\":[[536,6]]}                                                                                                                                 testdata4dat2data23datax".toCharArray();
        AssertStringEquals(expected, str, failures);

        // 5
        success = KeyValueRemove(disk, "test".toCharArray(), message);

        AssertTrue(success, failures);

        data = new ByteArrayReference();
        ReadRange(disk, 0d, disk.entrysize + 2d*directorySize + ByteArrayLength(value) + ByteArrayLength(value2) + ByteArrayLength(value3), data);
        str = UTF8ToUTF16(data.bytes);

        expected = "{\"DirectorySize\":200,\"ActiveDirectory\":128,\"FreePosition\":552}                                                                  {\"keys\":{\"d3\":[546,6],\"test3\":[536,4]},\"free\":[[528,14]]}                                                                                                                                               {\"keys\":{\"test\":[528,8],\"d3\":[546,6],\"test3\":[536,4]},\"free\":[[536,6]]}                                                                                                                                 testdata4dat2data23datax".toCharArray();
        AssertStringEquals(expected, str, failures);

        // 6
        value6 = UTF16ToUTF8("newval".toCharArray());
        success = KeyValueUpdate(disk, "test3".toCharArray(), value6, message);

        AssertTrue(success, failures);

        data = new ByteArrayReference();
        ReadRange(disk, 0d, disk.entrysize + 2d*directorySize + ByteArrayLength(value) + ByteArrayLength(value2) + ByteArrayLength(value3), data);
        str = UTF8ToUTF16(data.bytes);

        expected = "{\"DirectorySize\":200,\"ActiveDirectory\":328,\"FreePosition\":552}                                                                  {\"keys\":{\"d3\":[546,6],\"test3\":[536,4]},\"free\":[[528,14]]}                                                                                                                                               {\"keys\":{\"d3\":[546,6],\"test3\":[528,6]},\"free\":[[528,12]]}                                                                                                                                               newvalta4dat2data23datax".toCharArray();
        AssertStringEquals(expected, str, failures);
    }

    public static void TestKeyValue3(NumberReference failures) {
        DiskStructure disk;
        ByteArrayReference data;
        char [] str, expected;
        ByteArray value, value2, value3;
        StringReference message;
        boolean success;
        double directorySize;

        message = new StringReference();

        disk = new DiskStructure();
        disk.entries = 1000d;
        disk.entrysize = 128d;
        disk.data = CreateByteArray(disk.entries * disk.entrysize);
        directorySize = 200d;

        success = InitializeKeyValueStore(disk, directorySize);
        AssertTrue(success, failures);

        data = new ByteArrayReference();
        ReadRange(disk, 0d, disk.entrysize + 2d*directorySize, data);
        str = UTF8ToUTF16(data.bytes);

        expected = "{\"DirectorySize\":200,\"ActiveDirectory\":128,\"FreePosition\":528}                                                                  {\"keys\":{},\"free\":[]}                                                                                                                                                                                   {\"keys\":{},\"free\":[]}                                                                                                                                                                                   ".toCharArray();
        AssertStringEquals(expected, str, failures);

        // 1
        value = UTF16ToUTF8("testdata".toCharArray());
        success = KeyValueAdd(disk, "test".toCharArray(), value, message);

        AssertTrue(success, failures);

        data = new ByteArrayReference();
        ReadRange(disk, 0d, disk.entrysize + 2d*directorySize + ByteArrayLength(value), data);
        str = UTF8ToUTF16(data.bytes);

        expected = "{\"DirectorySize\":200,\"ActiveDirectory\":328,\"FreePosition\":536}                                                                  {\"keys\":{},\"free\":[]}                                                                                                                                                                                   {\"keys\":{\"test\":[528,8]},\"free\":[]}                                                                                                                                                                     testdata".toCharArray();
        AssertStringEquals(expected, str, failures);

        // 2
        value2 = UTF16ToUTF8("test2data2".toCharArray());
        success = KeyValueAdd(disk,"test2".toCharArray(), value2, message);

        AssertTrue(success, failures);

        data = new ByteArrayReference();
        ReadRange(disk, 0d, disk.entrysize + 2d*directorySize + ByteArrayLength(value) + ByteArrayLength(value2), data);
        str = UTF8ToUTF16(data.bytes);

        expected = "{\"DirectorySize\":200,\"ActiveDirectory\":128,\"FreePosition\":546}                                                                  {\"keys\":{\"test\":[528,8],\"test2\":[536,10]},\"free\":[]}                                                                                                                                                    {\"keys\":{\"test\":[528,8]},\"free\":[]}                                                                                                                                                                     testdatatest2data2".toCharArray();
        AssertStringEquals(expected, str, failures);

        // 3
        value3 = UTF16ToUTF8("3datax".toCharArray());
        success = KeyValueAdd(disk,"d3".toCharArray(), value3, message);

        AssertTrue(success, failures);

        data = new ByteArrayReference();
        ReadRange(disk, 0d, disk.entrysize + 2d*directorySize + ByteArrayLength(value) + ByteArrayLength(value2) + ByteArrayLength(value3), data);
        str = UTF8ToUTF16(data.bytes);

        expected = "{\"DirectorySize\":200,\"ActiveDirectory\":328,\"FreePosition\":552}                                                                  {\"keys\":{\"test\":[528,8],\"test2\":[536,10]},\"free\":[]}                                                                                                                                                    {\"keys\":{\"test\":[528,8],\"test2\":[536,10],\"d3\":[546,6]},\"free\":[]}                                                                                                                                       testdatatest2data23datax".toCharArray();
        AssertStringEquals(expected, str, failures);

        // 4
        success = KeyValueRemove(disk, "test".toCharArray(), message);

        AssertTrue(success, failures);

        data = new ByteArrayReference();
        ReadRange(disk, 0d, disk.entrysize + 2d*directorySize + ByteArrayLength(value) + ByteArrayLength(value2) + ByteArrayLength(value3), data);
        str = UTF8ToUTF16(data.bytes);

        expected = "{\"DirectorySize\":200,\"ActiveDirectory\":128,\"FreePosition\":552}                                                                  {\"keys\":{\"test2\":[536,10],\"d3\":[546,6]},\"free\":[[528,8]]}                                                                                                                                               {\"keys\":{\"test\":[528,8],\"test2\":[536,10],\"d3\":[546,6]},\"free\":[]}                                                                                                                                       testdatatest2data23datax".toCharArray();
        AssertStringEquals(expected, str, failures);

        // 4
        success = KeyValueRemove(disk, "test2".toCharArray(), message);

        AssertTrue(success, failures);

        data = new ByteArrayReference();
        ReadRange(disk, 0d, disk.entrysize + 2d*directorySize + ByteArrayLength(value) + ByteArrayLength(value2) + ByteArrayLength(value3), data);
        str = UTF8ToUTF16(data.bytes);

        expected = "{\"DirectorySize\":200,\"ActiveDirectory\":328,\"FreePosition\":552}                                                                  {\"keys\":{\"test2\":[536,10],\"d3\":[546,6]},\"free\":[[528,8]]}                                                                                                                                               {\"keys\":{\"d3\":[546,6]},\"free\":[[528,18]]}                                                                                                                                                               testdatatest2data23datax".toCharArray();
        AssertStringEquals(expected, str, failures);

        // 5
        success = KeyValueRemove(disk, "d3".toCharArray(), message);

        AssertTrue(success, failures);

        data = new ByteArrayReference();
        ReadRange(disk, 0d, disk.entrysize + 2d*directorySize + ByteArrayLength(value) + ByteArrayLength(value2) + ByteArrayLength(value3), data);
        str = UTF8ToUTF16(data.bytes);

        expected = "{\"DirectorySize\":200,\"ActiveDirectory\":128,\"FreePosition\":528}                                                                  {\"keys\":{},\"free\":[]}                                                                                                                                                                                   {\"keys\":{\"d3\":[546,6]},\"free\":[[528,18]]}                                                                                                                                                               testdatatest2data23datax".toCharArray();
        AssertStringEquals(expected, str, failures);
    }

    public static void TestKeyValueLarge(NumberReference failures) {
        DiskStructure disk;
        char [] key, value;
        ByteArray valueData;
        StringReference message;
        boolean success;
        double directorySize, i, diskFree1, dirFree1, diskFree2, dirFree2, diskFree3, dirFree3;
        NumberReference diskSpace, directorySpace;
        ByteArrayReference valueRef;

        message = new StringReference();

        disk = new DiskStructure();
        disk.entries = 32d * 1024d;
        disk.entrysize = 4096d;
        disk.data = CreateByteArray(disk.entries * disk.entrysize);
        directorySize = 6d * 4096d;

        success = InitializeKeyValueStore(disk, directorySize);
        AssertTrue(success, failures);

        diskSpace = new NumberReference();
        directorySpace = new NumberReference();
        KeyValueFree(disk, diskSpace, directorySpace, message);
        diskFree1 = diskSpace.numberValue;
        dirFree1 = diskSpace.numberValue;

        for(i = 0d; i < 300d; i = i + 1d){
            key = nCreateStringDecimalFromNumber(i);
            value = nCreateStringDecimalFromNumber(i);
            valueData = UTF16ToUTF8(value);
            success = KeyValueAdd(disk, key, valueData, message);

            AssertTrue(success, failures);
        }

        KeyValueFree(disk, diskSpace, directorySpace, message);
        diskFree2 = diskSpace.numberValue;
        dirFree2 = diskSpace.numberValue;

        AssertTrue(diskFree1 > diskFree2, failures);
        AssertTrue(dirFree1 > dirFree2, failures);

        for(i = 0d; i < 300d; i = i + 1d){
            key = nCreateStringDecimalFromNumber(i);
            valueRef = new ByteArrayReference();
            success = KeyValueRead(disk, key, valueRef, message);
            value = UTF8ToUTF16(valueRef.bytes);

            AssertTrue(success, failures);
            AssertStringEquals(key, value, failures);
        }

        for(i = 0d; i < 300d; i = i + 1d){
            key = nCreateStringDecimalFromNumber(i);
            success = KeyValueRemove(disk, key, message);

            AssertTrue(success, failures);
        }

        KeyValueFree(disk, diskSpace, directorySpace, message);
        diskFree3 = diskSpace.numberValue;
        dirFree3 = diskSpace.numberValue;

        AssertEquals(diskFree1, diskFree3, failures);
        AssertEquals(dirFree1, dirFree3, failures);
    }
}







